package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.item.TotemItem;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.network.PacketBuffer;
import net.minecraft.util.Hand;
import net.minecraftforge.fml.network.NetworkEvent;

import java.util.function.Supplier;

public class ChargeTotemPacket {
	private final Hand hand;
	
	public ChargeTotemPacket(final Hand hand) {
		this.hand = hand;
	}
	
	public static void encode(final ChargeTotemPacket packet, final PacketBuffer buffer) {
		Network.writeVersionInfo(buffer, false);
		buffer.writeEnum(packet.hand);
	}
	
	public static ChargeTotemPacket decode(final PacketBuffer buffer) {
		Network.checkVersion(buffer);
		return new ChargeTotemPacket(buffer.readEnum(Hand.class));
	}
	
	public static void handle(final ChargeTotemPacket packet, final Supplier<NetworkEvent.Context> contextSupplier) {
		final NetworkEvent.Context context = contextSupplier.get();
		context.enqueueWork(() -> {
			ServerPlayerEntity sender = context.getSender();
			if (sender == null) { return; }
			
			ItemStack totemStack = (packet.hand.equals(Hand.MAIN_HAND)) ? sender.getMainHandItem() : sender.getOffhandItem();
			ItemStack chargeStack = (packet.hand.equals(Hand.MAIN_HAND)) ? sender.getOffhandItem() : sender.getMainHandItem();
			int charge = TotemItem.getCharge(totemStack);
			
			if (charge >= TotemItem.getMaxCharge(totemStack)) {
				return;
			}
			
			TotemItem.setCharge(totemStack, charge + 1);
			chargeStack.setCount(chargeStack.getCount() - 1);
		});
		context.setPacketHandled(true);
	}
}
